#ifndef ANIMATION_H
#define ANIMATION_H
/*
	Animadead: A Skeletal Animation Library
	Copyright (C) 2005 John C. Butterfield

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	John C. Butterfield
	johnb003@hotmail.com
*/

#include <vector>
#include <string>
#include <map>

#include "skeleton.h"

namespace ad
{
	// pretty useless class right now
	struct Frame
	{
		Pose *pose; // dynamic

		Frame();
		~Frame();
	};

	// a collection of sampled frames making up an animation
	struct Anim
	{
		Hierarchy *bones; // dynamic
		unsigned int numFrames;
		Frame *frames; // array
		float fps;

		Anim(std::string filename);				// loads a .jba file
		~Anim();
	};

	// loads an animation and keeps track of the instances of previously loaded animations.
	class AnimCacher
	{
		AnimCacher(); // singleton, use MeshCacher::instance() to use it.
		static AnimCacher *inst; // dynamic
		std::map<std::string, Anim *> cache;

	public:
		~AnimCacher();

		static AnimCacher *instance();
		void release();
		Anim *GetAnim(std::string filename);	// returns instance of loaded .jba file, or loads it if it isn't already.
	};

	// needed to play a single instance of an animation at different frames.
	class AnimPlayer : public BlendBase
	{
		float frame;
		float weight;
		unsigned int keyframe;
//		Vector3 *currentPosition;
//		Vector3 *lastPosition;

	public:
		Anim *anim; // reference

		AnimPlayer(Anim *anim, float startTime = 0);
		~AnimPlayer();
//		void GetKeyframeVelocity(Vector3 *tempVel);
//		void GetVelocity(Vector3 *tempVel);
		void SetFrame(float frametime);
		void Update(float time);
		void Calculate(Pose *out_pose, HiBoneMask *optionalMask = 0);
//		void Update(unsigned int framesPassed);
	};

}

#endif // ANIMATION_H
