#ifndef MATRIXMATH_H
#define MATRIXMATH_H
/*
	Animadead: A Skeletal Animation Library
	Copyright (C) 2005 John C. Butterfield

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	John C. Butterfield
	johnb003@hotmail.com
*/

#include "quaternion.h"
#include "vector3.h"

namespace ad
{
	class Matrix
	{
	// column-major (Like OpenGL)
	//  0  4  8 12
	//  1  5  9 13
	//  2  6 10 14
	//  3  7 11 15
	
	public:
		float m[16];

		Matrix();
		Matrix(const float farray[16]);
		Matrix(const Quaternion &q, float x=0, float y=0, float z=0);
		Matrix(const Matrix &m);
		Vector3 Rotate(const Vector3 &v);
		void Set(const Quaternion &q, float x=0, float y=0, float z=0);
		
    	friend Vector3 operator*(const Matrix &lhs, const Vector3 &rhs);
    	friend Matrix operator*(const Matrix &lhs, const Matrix &rhs);
	};
}

#endif // MATRIXMATH_H
