#ifndef MODEL_H
#define MODEL_H
/*
	Animadead: A Skeletal Animation Library
	Copyright (C) 2005 John C. Butterfield

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	John C. Butterfield
	johnb003@hotmail.com
*/

#include <vector>
#include <string>

#include "animation.h"
#include "mesh.h"

namespace ad
{
	class Model
	{
	public:
		enum Type {
			STATIC,
			DEFORMABLE
		};

	private:
		MeshCacher *meshCacher;
		AnimCacher *animCacher;
		// a model can be made up of several meshes
		std::vector<AnimPlayer *> anims;
		std::vector<BlendBase *> blends;
		std::vector<HiBoneMask *> masks;

	protected:
		std::vector<Mesh *> meshes;
//		float timeweight;
//		AnimPlayer *velocityControl;
//		std::vector<unsigned int> animFromBone;
		bool originSubtraction;
		UserFunction *fp;
		Model::Type type;
		Anim *arbitraryAnim; // move me back to protected

	public:
		Pose *renderPose;
		Matrix *worldBones;
		Matrix *worldBonesTEMP;

		Model(Type type, UserFunction *texture_loader = 0);
		virtual ~Model();

		// needed because when constructed, the class doesn't know about any animations
		// and therefore, cannot determine how many bones to make the renderPose
		void AllocateRenderPose();
		unsigned int AddMesh(std::string filename);
		void SwapMesh(unsigned int index, std::string filename);
		AnimPlayer *AddAnim(std::string filename, float startFrame = 0);
		Blend *CreateBlend(BlendBase *from = 0, BlendBase *to = 0);
		HiBoneMask *CreateMask(const std::string &selections);
		void GenerateRenderPoseMatrices();
		
//		void SwapAnim(unsigned int index, std::string filename, unsigned int startFrame = 0);
//		unsigned int GetKeyframe(unsigned int anim);
//		void GetTransform(unsigned int bone, Matrix &matrix);
//		void Bind(unsigned int bone, unsigned int anim);
		void MovingOrigin(bool);
//		void OriginAnim(unsigned int index);
//		void GetKeyframeVelocity(Vector3 *velocity);
//		void GetVelocity(Vector3 *velocity);
		virtual void Update(float timestep);		// time in seconds.
//		void UpdateMeshes();
	};
}

#endif // MODEL_H
