#ifndef SKELETON_H
#define SKELETON_H
/*
	Animadead: A Skeletal Animation Library
	Copyright (C) 2005 John C. Butterfield

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	John C. Butterfield
	johnb003@hotmail.com
*/

#include "vector3.h"
#include "quaternion.h"

namespace ad
{
	struct HiBone
	{
		std::string name;
		int parent;
	};
	
	struct Hierarchy
	{
		HiBone *nodes; // array
		unsigned int numBones;
		
		Hierarchy(unsigned int numbones);
		~Hierarchy();
	};

	// Hierarchial Bone Selection Mask
	// Can be used to determine the entire skeleton, and a subset of that.
	class HiBoneMask
	{
	public:
		bool *mask; // array
		// the hierarchy can be from any animation, assuming they are all valid
		// (have the same skeleton and hierarchy)
		HiBoneMask(Hierarchy *hierarchy, const std::string &selections);
		~HiBoneMask();
	};
	
	// used to represent the orientation of each bone
	struct Transform
	{
		float x,y,z;
		Quaternion q;
	};
	
	struct Pose
	{
		unsigned int numBones;
		Transform *bones; // array
		Transform *root; // reference: bones[0]
		Vector3 keyframeVelocity; // this vector represents the offset of the root bone from the previous frame to the current

		Pose(unsigned int numBones);
		~Pose();

		void Slerp(const Pose &from, const Pose &to, float weight, HiBoneMask *optionalMask = 0);
	};

	// Blend Classes
	class BlendBase
	{
	public:
		// if no optional mask is used, the Calculation should calculate the pose for all bones
		virtual void Calculate(Pose *out_pose, HiBoneMask *optionalMask = 0) = 0;
		virtual ~BlendBase() {}
	};
	
	class Blend : public BlendBase
	{
		Pose *from_pose, *to_pose;
	public:
		// user parameters
		float weight;
		BlendBase *from, *to;

		Blend(unsigned int numBones, BlendBase *from = 0, BlendBase *to = 0);
		~Blend();
		
		void Calculate(Pose *out_pose, HiBoneMask *optionalMask = 0);
	};
}

#endif // SKELETON_H
